'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.determineUnambiguousAlias = determineUnambiguousAlias;
exports.toObjectOperations = toObjectOperations;

var _operations = require('./operations');

var _debug = require('../debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// determineUnambiguousAlias derives an alias from an expression. An alias is a name
// which is, unambiguously, the first name addressed by an expression. The following
// expressions all return "foo" as the alias:
//
//   foo
//   foo[]
//   foo->
//   foo[]->
//   foo[bar == 1]
//   foo[0]
//   foo[0..10]
//
// More complicated expressions such as foo.bar where the name is ambiguous are
// not allowed.
function determineUnambiguousAlias(operation) {
  (0, _debug2.default)('determineUnambiguousAlias()', operation);
  switch (operation.op) {
    case 'accessor':
      if (operation.path.length == 1) {
        if (operation.path[0].op == 'attribute') {
          return operation.path[0].name;
        }
      }
      break;
    case 'pipe':
      // Pipelines may specify an explicit alias (to preserve aliasing information in case of severe rewritage
      // during planning)
      if (operation.alias) {
        return operation.alias;
      }
      if (operation.path.length == 2) {
        const lastOpType = operation.path[1].op;
        if (lastOpType == 'filter' || lastOpType == 'subscript') {
          return determineUnambiguousAlias(operation.path[0]);
        }
      }
      break;
  }
}

function toObjectOperations(expressions) {
  // TODO: Convert a list of expressions to object operations

  return expressions.map(expr => {
    const alias = determineUnambiguousAlias(expr);
    if (alias) {
      (0, _debug2.default)('=> alias:', alias);
      return new _operations.Assignment(alias, expr);
    }
    (0, _debug2.default)('no  alias');
    return expr;
  });
}
//# sourceMappingURL=objectExpressionOps.js.map