'use strict';

var _parse = require('../parse');

var _parse2 = _interopRequireDefault(_parse);

var _plan = require('../plan');

var _plan2 = _interopRequireDefault(_plan);

var _exec = require('../exec');

var _exec2 = _interopRequireDefault(_exec);

var _debug = require('../debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

// Entry point for query engine. Accepts a GROQ-query + a fetcher to retrieve
// data from the data source, parses, plans out and then executes the query.
// The fetcher is a query that will receive a filter expression and is expected
// to return a promise that resolves to an array of documents. The fetcher must
// provide all documents matching the filter expression, but may provide false
// positives as all documents are double checked against the filter in the
// executor. That means: a fetcher may actually return the entire database
// every time and still be compliant, although potentially slow.

module.exports = (() => {
  var _ref = _asyncToGenerator(function* (options) {
    if (typeof options !== 'object') {
      throw new Error('query() takes a singele object as parameter. {source, params, fetcher, globalFilter}');
    }
    const source = options.source,
          params = options.params,
          fetcher = options.fetcher,
          globalFilter = options.globalFilter;


    (0, _debug2.default)("query:", source, "params:", params, "fetcher:", fetcher);
    const ast = (0, _parse2.default)(source, params);
    (0, _debug2.default)(JSON.stringify(ast));

    const globalFilterAst = globalFilter ? (0, _parse2.default)(globalFilter, {}) : null;

    const operations = (0, _plan2.default)(ast, globalFilterAst);
    (0, _debug2.default)(JSON.stringify(operations, null, 2));
    // fetcher = (filter) => {
    //   debug("fetch:", filter)
    //   return new Promise((resolve, reject) => {
    //     resolve([
    //       {
    //         a: 1,
    //         b: "one"
    //       },
    //       {
    //         a: 2,
    //         b: "two"
    //       }
    //     ])
    //   })
    // }
    const resultScope = yield (0, _exec2.default)({
      operations,
      fetcher
    });

    return resultScope.value;
  });

  function query(_x) {
    return _ref.apply(this, arguments);
  }

  return query;
})();
//# sourceMappingURL=index.js.map