import Scope from './Scope'
import debug from '../debug'

// Coerce a value to be on the comparable form. E.g. rewrites path("a.b.c") => "a.b.c"
function coerce(value) {
  if (value === null || value === undefined) {
    return null
  }
  if (value.toJSON) {
    return value.toJSON()
  }
  return value
}

const TYPE_ORDER = {
  'number': 1,
  'string': 2,
  'boolean': 3,
  'object': 4
}

function compare(a, b, ascending) {
  if (a === b) {
    return 0
  }
  if (typeof a !== typeof b) {
    if (TYPE_ORDER[typeof a] < TYPE_ORDER[typeof b]) {
      return -1
    }
    return 1
  }
  if (a < b) {
    return ascending ? -1 : 1
  }
  return ascending ? 1 : -1
}


export default function sort(scope, terms) {
  debug("sort()", scope, terms)
  terms = terms.slice()
  // Map each object into terms sorted by cardinality
  // so [{a: 1, b: "a"}, {a: 2, b: "b"}]|order(a, b)
  // becomes [[1, "a"], [2, "b"]]
  const arrayValue = Array.isArray(scope.value) ? scope.value : [scope.value]
  const itemWithTerms = arrayValue.map(item => ({
    rawValue: item,
    terms: terms.map(term => {
      return (new Scope({
        value: item
      })).resolveAccessor(term.expression.path).value
    })
  }))
  const termIsAscending = terms.map(term => term.direction == 'asc')

  const sorted = itemWithTerms.sort((a, b) => {
    for (let i = 0; i < terms.length; i++) {
      const ascending = termIsAscending[i]
      const comparision = compare(a.terms[i], b.terms[i], ascending)
      if (comparision !== 0) {
        return comparision
      }
    }
    return 0
  })

  return scope.clone({
    value: sorted.map(item => item.rawValue)
  })
}