import parse from '../src/parse'
import plan from '../src/plan'

import fs from 'fs'
import glob from 'glob'
import path from 'path'
import yaml from 'js-yaml'
import query from '../src/query'

describe('Reference queries', () => {
  const suites = glob.sync(path.join(__dirname, 'reference_queries', '*.yml')).map(filename => {
    try {
      const yamlSrc = fs.readFileSync(filename, {
        encoding: "UTF8"
      })
      return yaml.safeLoad(yamlSrc)
    } catch (error) {
      console.error(`Error while parsing ${filename}`)
      throw error
    }
  })

  Promise.all(suites.map(async suite => {
    await runSuite(suite)
  }))

})

async function runSuite(suite) {
  describe(suite.title, () => {
    suite.tests.forEach(test => {
      it(test.title, async () => {
        console.log(test.title, ":", test.query)
        const rootNode = parse(test.query)
        console.log("AST", JSON.stringify(rootNode, null, 2))
        const result = plan(rootNode)
        console.log("PLAN", JSON.stringify(result, null, 2))
      })
    })
  })
}